@@ -68,6 +68,14 @@ int mutils_munlock(__const void *addr, __const mutils_word32 len)
 	return(ret);
 }
 
+/*
+ * NOTE: In order for memory locking to be useful, we need our own malloc
+ * implementation, as we cannot guarantee that the system-supplied one will
+ * let us know how much memory has been malloc()'ed. For much the same reason,
+ * neither free() nor realloc() can zero safely. This allows the potential
+ * leakage of information.
+ */
+
 WIN32DLL_DEFINE
 void *
 mutils_malloc(__const mutils_word32 n)
@@ -89,6 +97,31 @@ mutils_malloc(__const mutils_word32 n)
 	return(ptr);
 }
 
+WIN32DLL_DEFINE
+void *
+mutils_realloc(__const void *ptr, __const mutils_word32 n)
+{
+	void *ptrOut;
+
+	if ((ptr == NULL) && (n > 0))
+	{
+		ptrOut = mutils_malloc(n);
+	}
+	else
+	{
+		if ((ptr != NULL) && (n == 0))
+		{
+			mutils_free(ptr);
+			ptrOut = NULL;
+		}
+		else
+		{
+		  ptrOut = realloc((void *) ptr, n);
+		}
+	}
+	return(ptrOut);
+}
+
 WIN32DLL_DEFINE
 void
 mutils_free(__const void *ptr)
@@ -373,6 +406,9 @@ WIN32DLL_DEFINE
 mutils_word8 *
 mutils_strcat(mutils_word8 *dest, __const mutils_word8 *src)
 {
+	mutils_word8 *ptrIn = src;
+	mutils_word8 *ptrOut = dest + mutils_strlen(dest);
+
 	if (dest == NULL)
 	{
 		return(NULL);
@@ -380,8 +416,15 @@ mutils_strcat(mutils_word8 *dest, __const mutils_word8 *src)
 	if (src == NULL)
 	{
 		return(dest);
+	}	
+	while (*src != 0)
+	{
+		*ptrOut = *ptrIn;
+		ptrOut++;
+		ptrIn++;
 	}
-	return((mutils_word8 *) strcat((char *) dest, (char *) src));
+	*ptrOut = '\0';
+	return((mutils_word8 *) dest);
 }
 
 WIN32DLL_DEFINE
@@ -464,9 +507,8 @@ mutils_word8
 mutils_val2char(mutils_word8 x)
 {
 	mutils_word8 out;
-	mutils_word8 out2;
 
-	static mutils_word8 *table = "0123456789abcdef";
+	static mutils_word8 *table = (mutils_word8 *) "0123456789abcdef";
 
 	out = *(table + x);
 
@@ -497,8 +539,6 @@ mutils_thequals(mutils_word8 *text, mutils_word8 *hash, __const mutils_word32 le
 	mutils_word8  *ptrText = text;
 	mutils_word8  *ptrHash = hash;
 	mutils_word32  loop;
-	mutils_word8   temp;
-	mutils_boolean equals;
 
 	for (loop = 0; loop < len; loop++, ptrHash++)
 	{
